@@ -1,30 +1,29 @@
 /*
- * Copyright (C) 2005-2017   Christopher C. Hulbert
- *
+ * Copyright (c) 2005-2018, Christopher C. Hulbert
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
  *
- *    1. Redistributions of source code must retain the above copyright notice,
- *       this list of conditions and the following disclaimer.
+ * 1. Redistributions of source code must retain the above copyright notice, this
+ *    list of conditions and the following disclaimer.
  *
- *    2. Redistributions in binary form must reproduce the above copyright
- *       notice, this list of conditions and the following disclaimer in the
- *       documentation and/or other materials provided with the distribution.
+ * 2. Redistributions in binary form must reproduce the above copyright notice,
+ *    this list of conditions and the following disclaimer in the documentation
+ *    and/or other materials provided with the distribution.
  *
- * THIS SOFTWARE IS PROVIDED BY CHRISTOPHER C. HULBERT ``AS IS'' AND ANY
- * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
- * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED. IN NO EVENT SHALL CHRISTOPHER C. HULBERT OR CONTRIBUTORS BE
- * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+ * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
+ * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
+ * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
+ * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
+
 #include <stdlib.h>
 #include <stdarg.h>
 #include <stdio.h>
@@ -69,16 +68,17 @@ static const char *helpstr[] = {
 "mat_file          name of the MAT file to dump",
 "var1 var2 ...     If specified, dumps only listed variables",
 "",
-"Report bugs to <chulbe2lsu@sourceforge.net>.",
+"Report bugs to <t-beu@users.sourceforge.net>.",
 NULL
 };
 
-static const char *mxclass[15] = { "mxCELL_CLASS", "mxSTRUCT_CLASS", "mxOBJECT_CLASS",
-                             "mxCHAR_CLASS", "mxSPARSE_CLASS", "mxDOUBLE_CLASS",
-                             "mxSINGLE_CLASS", "mxINT8_CLASS", "mxUINT8_CLASS",
-                             "mxINT16_CLASS", "mxUINT16_CLASS", "mxINT32_CLASS",
-                             "mxUINT32_CLASS","mxINT64_CLASS","mxUINT64_CLASS"
-                            };
+static const char *mxclass[18] = { "mxUNKNOWN_CLASS",
+    "mxCELL_CLASS", "mxSTRUCT_CLASS", "mxOBJECT_CLASS",
+    "mxCHAR_CLASS", "mxSPARSE_CLASS", "mxDOUBLE_CLASS",
+    "mxSINGLE_CLASS", "mxINT8_CLASS", "mxUINT8_CLASS",
+    "mxINT16_CLASS", "mxUINT16_CLASS", "mxINT32_CLASS",
+    "mxUINT32_CLASS", "mxINT64_CLASS", "mxUINT64_CLASS",
+    "mxFUNCTION_CLASS", "mxOPAQUE_CLASS" };
 static int printdata = 0;
 static int human_readable = 0;
 static int print_whos_first = 1;
@@ -258,9 +258,9 @@ slab_select_valid(int rank,int *start,int *stride,int *edge,matvar_t *matvar)
         } else if ( *edge == -1 ) {
             /* index of the form 1:stride:end */
             *edge = nmemb;
-            *edge = floor((double)(*edge-*start-1) / (double)*stride)+1;
+            *edge = (int)floor((double)(*edge-*start-1) / (double)*stride)+1;
         } else if ( *edge > 0 ) {
-            *edge = floor((double)(*edge-*start-1) / (double)*stride)+1;
+            *edge = (int)floor((double)(*edge-*start-1) / (double)*stride)+1;
         }
         nmemb = *edge;
     } else {
@@ -282,7 +282,7 @@ slab_select_valid(int rank,int *start,int *stride,int *edge,matvar_t *matvar)
                     start[i] = 0;
             } else if ( edge[i] < 0 ) {
                 /* index of the form 1:stride:end */
-                edge[i] = floor((double)(matvar->dims[i]-start[i]-1) / (double)stride[i])+1;
+                edge[i] = (int)floor((double)(matvar->dims[i]-start[i]-1) / (double)stride[i])+1;
             } else if ( edge[i] > matvar->dims[i] ) {
                 /* edge can't be bigger than the size of the dimension */
                 fprintf(stderr,"edge out of bound on dimension %d",i+1);
@@ -300,7 +300,7 @@ slab_select_valid(int rank,int *start,int *stride,int *edge,matvar_t *matvar)
                 /* index of the form 3:3 */
                 edge[i] = 1;
             } else if ( edge[i] > 0 ) {
-                edge[i] = floor((double)(edge[i]-start[i]-1) / (double)stride[i])+1;
+                edge[i] = (int)floor((double)(edge[i]-start[i]-1) / (double)stride[i])+1;
             }
             nmemb *= edge[i];
         }
@@ -567,7 +567,7 @@ print_whos(matvar_t *matvar)
     } else {
         printf("  %10" SIZE_T_FMTSTR,nbytes);
     }
-    printf("  %-18s\n",mxclass[matvar->class_type-1]);
+    printf("  %-18s\n",mxclass[matvar->class_type]);
 
     return;
 }
@@ -680,6 +680,10 @@ static void
 print_default_numeric_3d(matvar_t *matvar)
 {
     size_t i, j, k, l, stride;
+
+    if ( NULL == matvar->data )
+        return;
+
     stride = Mat_SizeOf(matvar->data_type);
     if ( matvar->isComplex ) {
         mat_complex_split_t *complex_data = (mat_complex_split_t*)matvar->data;
@@ -868,7 +872,7 @@ main (int argc, char *argv[])
                 exit(EXIT_SUCCESS);
             case 'V':
                 printf("%s %s\nWritten by Christopher Hulbert\n\n"
-                       "Copyright(C) 2006-2017 Christopher C. Hulbert\n",
+                       "Copyright(C) 2006-2018 Christopher C. Hulbert\n",
                        prog_name,MATIO_VERSION_STR);
                 exit(EXIT_SUCCESS);
             default:
